game = gideros.class(Sprite)

require "box2d"

local clock = os.clock

score = 0

pause = false

function game: onEnterFrame()

	application:setBackgroundColor(0x00ff0)
	
  -- gravity
	if not pause then
		self.world:step(1/25, 8, 3)
	end
	
    --iterate through all child sprites
    for i = 1, self:getNumChildren() do
        --get specific sprite
        local sprite = self:getChildAt(i)
        -- check if sprite HAS a body (ie, physical object reference we added)
        if sprite.body then
            --update position to match box2d world object's position
            --get physical body reference
            local body = sprite.body
            --get body coordinates
            local bodyX, bodyY = body:getPosition()
            --apply coordinates to sprite
            sprite:setPosition(bodyX, bodyY)
            --apply rotation to sprite
            sprite:setRotation(body:getAngle() * 180 / math.pi)
        end
    end
	
end

function game:onExitBegin()
  self:removeEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
  self:removeEventListener(Event.ENTER_FRAME, self.pipemove)
  self:removeEventListener(Event.ENTER_FRAME, self.collide)
  self:removeEventListener(Event.ENTER_FRAME, self.scoreCount)
end

function game:init()

    self.world = b2.World.new(0, 10, true)
     
    --set up screen dimensions
    screenW = application:getContentWidth()
    screenH = application:getContentHeight()
     

	 
	--create background
	local bg = Sprite.new()
	local bgimg = Bitmap.new(Texture.new("Pictures/BG.png"))
	bg:setPosition(0,-50)
	bg:setScale(1,1.2)
	bg:addChild(bgimg)
	self:addChild(bg)
	 
	 --add barriers
    self:wall(0,screenH/2,10,screenH) --x,y,width,height --left
    self:wall(180,0,360,10) --top
    self:wall(screenW,screenH/2,10,screenH)--right
    self:wall(180,480,360,10)--bottom
	 
    --create the spud
    self:ball(100, 100)
	
	self:pipecreate(self)
	
	score = 0
	
	scoreText = TextField.new(nil,score)
	scoreText:setPosition(0,0+scoreText:getHeight())
	scoreText:setScale(4,4)
	self:addChild(scoreText)
	
	local res = dataSaver.loadValue("highscore")
	if res == nil then
		dataSaver.saveValue("highscore", 0)
	end
	hiscore = res
	
	hiscoreText = TextField.new(nil,hiscore)
	hiscoreText:setPosition(0,0)
	hiscoreText:setScale(4,4)
	self:addChild(hiscoreText)
	
	pausebutton = Bitmap.new(Texture.new("Pictures/Pause.png"))
	pausebutton:setAnchorPoint(0.5,0.5)
	pausebutton:setScale(0.3,0.3)
	pausebutton.x,pausebutton.y = (160),(15)
	pausebutton:setPosition(pausebutton.x,pausebutton.y)
	--pausebutton:setVisible(false)
	self:addChild(pausebutton)
	
	unpausebutton = Bitmap.new(Texture.new("Pictures/unpause.png"))
	unpausebutton:setAnchorPoint(0.5,0.5)
	unpausebutton:setScale(0.3,0.3)
	unpausebutton.x,unpausebutton.y = (160),(15)
	unpausebutton:setPosition(unpausebutton.x,unpausebutton.y)
	unpausebutton:setVisible(false)
	self:addChild(unpausebutton)
	
	stage:addEventListener(Event.APPLICATION_SUSPEND, function()
    dataSaver.saveValue("highscore", hiscore )
	end)
	
	stage:addEventListener(Event.APPLICATION_EXIT, function()
    dataSaver.saveValue("highscore", hiscore )
	end)
	
	self:addEventListener(Event.ENTER_FRAME, self.scoreCount)
	self:addEventListener(Event.ENTER_FRAME, self.collide)
	self:addEventListener(Event.MOUSE_UP, pausegame)
    self:addEventListener(Event.ENTER_FRAME, self.onEnterFrame, self)
    self:addEventListener("exitBegin", self.onExitBegin, self)
end


function game:pipecreate(s)

	self:pipe(1000,250,0)
	self:pipe2(1060,100,180)

end

function force(self)
	if not pause then
		self:applyLinearImpulse(0, -10, ball:getX(), ball:getY())
	end
end

function game:ball(x, y)

    ball = Bitmap.new(Texture.new("Pictures/potatoTrans.png"))
    ball:setAnchorPoint(0.5,0.5)
     
    ball:setPosition(x,y)
	
	ball:setScale(0.5,0.5)
     
    --get radius
    local radius = ball:getWidth()/2
     
    --create box2d physical object
    body = self.world:createBody{type = b2.DYNAMIC_BODY}
    body:setPosition(ball:getX(), ball:getY())
    body:setAngle(ball:getRotation() * math.pi/180)
    local circle = b2.CircleShape.new(0, 0, radius)
    local fixture = body:createFixture{shape = circle, density = 1.0, 
    friction = 0.1, restitution = 0}
    ball.body = body
    ball.body.type = "ball"
	
	self:addEventListener(Event.MOUSE_DOWN, force, ball.body)
	 
    --add to scene
    self:addChild(ball)
	 
    --return created object
    return ball
end

function game:wall(x, y, width, height)
    local wall = Shape.new()
    wall:beginPath()
    
	--create shape
    wall:moveTo(-width/2,-height/2)
    wall:lineTo(width/2, -height/2)
    wall:lineTo(width/2, height/2)
    wall:lineTo(-width/2, height/2)
    wall:closePath()
    wall:endPath()
    wall:setPosition(x,y)
    
	--create body
    local body = self.world:createBody{type = b2.STATIC_BODY}
    body:setPosition(wall:getX(), wall:getY())
    body:setAngle(wall:getRotation() * math.pi/180)
    local poly = b2.PolygonShape.new()
    poly:setAsBox(wall:getWidth()/2, wall:getHeight()/2)
    local fixture = body:createFixture{shape = poly, density = 1.0, 
    friction = 0.1, restitution = 0}
    wall.body = body
    wall.body.type = "wall"
    
    self:addChild(wall)
    
    return wall
end

music=Sound.new("Sounds/Monkey-Island-Band.mp3")

function restartMusic()
	playingMusic:removeEventListener(Event.COMPLETE,restartMusic)
	playingMusic=music:play()
	if playingMusic then
		playingMusic:addEventListener(Event.COMPLETE,restartMusic)
	end
end

playingMusic=music:play()

if playingMusic then
	playingMusic:addEventListener(Event.COMPLETE,restartMusic)
end

function pausegame (event)

	if pausebutton:hitTestPoint(event.x,event.y) then
		if not pause then --game isn't paused
			pauseswitch(true)
			pause = true
		elseif pause then --game is paused
			pauseswitch(false)
			pause = false
		end
	end

end

function pauseswitch(a)

	if a then
		unpausebutton:setVisible(true)
		pausebutton:setVisible(false)
	
	elseif not a then
		unpausebutton:setVisible(false)
		pausebutton:setVisible(true)
	end

end

function game:pipe(x,y,r)
	
	pipe = Bitmap.new(Texture.new("Pictures/ForkTrans.png"))
	pipe:setPosition(x,y)
	pipe:setScale(0.5,1)
	pipe:setRotation(r)
	self:addChild(pipe)
	
	pipe:addEventListener(Event.ENTER_FRAME,game.pipemove)
	
	return pipe
end

function game:pipe2(x,y,r)
	
	pipe2 = Bitmap.new(Texture.new("Pictures/ForkTrans.png"))
	pipe2:setPosition(x,y)
	pipe2:setScale(0.5,1)
	pipe2:setRotation(r)
	self:addChild(pipe2)
	
	pipe2:addEventListener(Event.ENTER_FRAME,game.pipemove)
	
	return pipe2
end

function game:pipemove()

	rand = math.random(150,400)
	
	x0 = pipe:getX()
	y0 = pipe:getY()
	if not pause then
	pipe:setPosition(x0-2,y0)
	end
	
	x1 = pipe2:getX()
	y1 = pipe2:getY()
	if not pause then
	pipe2:setPosition(x1-2,y1)
	end
	
	pipescrn = 576
	pipescrn2 = 60
	randgap = 150
	
	stuff = 50
	
	if x0 < -stuff then
		pipe:setPosition(pipescrn,rand)
		pipe2:setPosition(pipescrn + pipescrn2,rand - randgap)
	end
	
end

function game:collide()
	
	if ball:getY() > pipe:getY() - ball:getHeight()/2 then
		if pipe:getX() < 100 + ball:getWidth()/2 then
			if pipe:getX() > pipe:getWidth() then
				sceneManager:changeScene("results", 0.3, SceneManager.moveFromRight, easing.outBack)
				dataSaver.saveValue("highscore", hiscore)
			end
		end
	end
	
	if ball:getY() < pipe2:getY() + ball:getHeight()/2 then
		if pipe:getX() < 100 + ball:getWidth()/2 then
			if pipe:getX() > pipe:getWidth() then
				sceneManager:changeScene("results", 0.3, SceneManager.moveFromRight, easing.outBack)
				dataSaver.saveValue("highscore", hiscore)
			end
		end
	end

end

function game:scoreCount()
	derp=no
	if pipe:getX() == 40 then
		derp = yes
		score = score + 1
		scoreText:setText(score)
	end
	
	if pipe:getX() == 41 then
		if derp == no then
			score = score + 1
			scoreText:setText(score)
			derp = yes
		end
	end
	
	if pipe:getX() == 42 then
		if derp == no then
			score = score + 1
			scoreText:setText(score)
			derp = yes
		end
	end
	
	if score > hiscore then
		hiscore = score
	end
	
	hiscoreText:setText(hiscore)
	hiscoreText:setPosition(320 - hiscoreText:getWidth(),0+hiscoreText:getHeight())
	
	scoreText:setPosition(0,0+scoreText:getHeight())
end